"use strict";

var _bluebird = _interopRequireDefault(require("bluebird"));

var _sage_util = _interopRequireDefault(require("../../build/sage_util"));

var _async = _interopRequireDefault(require("async"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

module.exports = function (self, name, schema, sage) {
  self.destroy = function (options = {}) {
    const self = this;
    return new _bluebird.default((resolve, reject) => {
      const pk = this.get(this._schema.primaryKey);

      if (!pk) {
        sage.logger.warn('Missing primary key on destroy. Who do I destroy?');
        return reject('Missing primary key.');
      }

      const sql = sage.knex(this._name).where(this._schema.primaryKey, pk).del().toString();
      let connection;

      _async.default.series([// Get connection
      function (next) {
        sage.getConnection({
          transaction: options.transaction
        }).then(c => {
          connection = c;
          next();
        }).catch(err => {
          next(err);
        });
      }, // Perform operation
      function (next) {
        sage.logger.debug(sql);
        connection.execute(sql, (err, results) => {
          if (err) {
            sage.logger.error('Could not destroy.');
          }

          next(err);
        });
      }], err => {
        if (err) {
          sage.logger.error(err);
          return sage.afterExecute(connection).then(() => {
            reject(err);
          });
        }

        sage.afterExecuteCommitable(connection).then(() => {
          resolve();
        }).catch(reject);
      });
    });
  };
};