"use strict";

var _lodash = _interopRequireDefault(require("lodash"));

var _bluebird = _interopRequireDefault(require("bluebird"));

var _sage_util = _interopRequireDefault(require("../../build/sage_util"));

var _async = _interopRequireDefault(require("async"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

module.exports = function (self, name, schema, sage) {
  self.save = function (options = {}) {
    const self = this;
    return new _bluebird.default((resolve, reject) => {
      if (!this.get(this._schema.primaryKey)) {
        sage.logger.warn("No primary key. I don't know who to save.");
        return reject();
      }

      if (!this.valid) {
        sage.logger.warn('Invalid properties on model');
        return reject();
      } // save it to the database


      const pk = schema.primaryKey;

      const result = _sage_util.default.getUpdateSQL(this.dirtyProps);

      result.values = _sage_util.default.fixDateBug(this.schema, result.values); // Convert blob fields into buffers before saving

      _lodash.default.each(_lodash.default.keys(result.values), key => {
        if (this.normalized[key] && this.normalized[key].constructor.name === 'Buffer') {
          result.values[key] = this.normalized[key];
        }
      });

      let sql = `UPDATE ${name} SET ${result.sql} WHERE ${pk}=:${pk}`;
      sql = _sage_util.default.amendDateFields(this.schema, sql);
      sql = _sage_util.default.amendTimestampFields(this.schema, sql);
      result.values[pk] = this.get(pk);
      let connection;

      _async.default.series([// Get connection
      function (next) {
        sage.getConnection({
          transaction: options.transaction
        }).then(c => {
          connection = c;
          next();
        }).catch(next);
      }, // Perform operation
      function (next) {
        sage.logger.debug(sql, result.values);
        connection.execute(sql, result.values, (err, result) => {
          if (!err) {
            self.mergeProps();
          }

          next(err);
        });
      }], err => {
        if (err) {
          sage.logger.error(err);
          return sage.afterExecute(connection).then(() => {
            reject(err);
          });
        }

        sage.afterExecuteCommitable(connection).then(() => {
          if (err) {
            return reject(err);
          }

          return resolve();
        }).catch(reject);
      });
    });
  };
};