"use strict";

var _bluebird = _interopRequireDefault(require("bluebird"));

var _sage_util = _interopRequireDefault(require("../../build/sage_util"));

var _async = _interopRequireDefault(require("async"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

module.exports = function (modelClass, name, schema, sage) {
  modelClass.count = function (values = {}, options = {}) {
    const self = this;

    const result = _sage_util.default.getSelectANDSQL(values);

    let sql = `SELECT COUNT(*) FROM ${name}`;

    if (result.sql != '') {
      sql += ` WHERE ${result.sql}`;
    }

    let count;
    return new _bluebird.default((resolve, reject) => {
      let connection;

      _async.default.series([// Establish Connection
      function (next) {
        sage.getConnection({
          transaction: options.transaction
        }).then(c => {
          connection = c;
          return next();
        }).catch(next);
      }, // Perform operation
      function (next) {
        sage.logger.debug(sql, result.values);
        connection.execute(sql, result.values, (err, result) => {
          if (!err) {
            try {
              count = result.rows[0][0];
            } catch (e) {
              next(e);
            }
          }

          next(err);
        });
      }], err => {
        if (err) {
          sage.logger.error(err);
        }

        sage.afterExecute(connection).then(() => {
          if (err) {
            return reject(err);
          }

          return resolve(count);
        }).catch(reject);
      });
    });
  };
};