"use strict";

var _bluebird = _interopRequireDefault(require("bluebird"));

var _async = _interopRequireDefault(require("async"));

var _sage_util = _interopRequireDefault(require("../../build/sage_util"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

module.exports = function (modelClass, name, schema, sage) {
  modelClass.findOne = function (values = {}, options = {}) {
    const self = this;
    const pk = schema.primaryKey;

    const result = _sage_util.default.getSelectANDSQL(values);

    const sql = `select * from (
        select a.*, ROWNUM rnum from (
          SELECT ${self._selectAllStringStatic()} FROM ${name} WHERE ${result.sql} ORDER BY ${pk} DESC
        ) a where rownum <= 1
      ) where rnum >= 0`;
    let finalResult;
    return new _bluebird.default((resolve, reject) => {
      let connection;

      _async.default.series([// Get connection
      function (next) {
        sage.getConnection({
          transaction: options.transaction
        }).then(c => {
          connection = c;
          return next();
        }).catch(next);
      }, function (next) {
        sage.logger.debug(sql, values);
        connection.execute(sql, values).then(result => _sage_util.default.resultToJSON(result, schema)).then(results => {
          if (results.length) {
            finalResult = new self(results[0], name, schema);
          }

          next();
        }).catch(next);
      }], err => {
        if (err) {
          sage.logger.error(err);
        }

        sage.afterExecute(connection).then(() => {
          if (err) {
            return reject(err);
          }

          return resolve(finalResult);
        }).catch(reject);
      });
    });
  };
};