"use strict";

var _lodash = _interopRequireDefault(require("lodash"));

var _sage_util = _interopRequireDefault(require("../../build/sage_util"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const knex = require('knex')({
  client: 'oracle'
});

class SelectQuery {
  constructor(sage, tableName, model, columns) {
    this.sage = sage;
    this.knex = knex(tableName);
    this.model = model; // needed to convert results into models

    this.knex.select(columns || '*');

    this.knex.exec = (options = {}) => this.exec(options);

    this.knex.execWithBindParams = (bindParams, options) => this.execWithBindParams(bindParams, options);

    return this.knex;
  }

  execWithBindParams(bindParams, options) {
    return this.exec(options, bindParams);
  }

  exec(options = {}, bindParams = []) {
    const models = [];
    const self = this;
    let connection;
    return self.sage.getConnection({
      transaction: options.transaction
    }).then(c => {
      connection = c;
    }).then(() => {
      let sql = self.knex.toString(); // Fix: [Error: ORA-01756: quoted string not properly terminated]

      sql = sql.replace(/\\'/g, "''");
      self.sage.logger.debug(sql);
      return connection.execute(sql, bindParams, options);
    }).then(result => _sage_util.default.resultToJSON(result, this.schema)).then(results => {
      _lodash.default.each(results, result => {
        models.push(new self.model(result));
      });

      return models;
    }).catch(err => {
      if (err) {
        self.sage.logger.error(err);
      }

      throw err;
    }).finally(() => self.sage.afterExecute(connection));
  }

}

module.exports = SelectQuery;